use std::path::Path;
use std::{fmt::Display, path::PathBuf};

use crate::error::Error;

use super::Section;

pub type BSI<'iter, I> = Box<dyn Iterator<Item = Result<Section<I>, Error>> + 'iter>;
/// Contains all the needed getters to construct a final
/// mushed and relocated executable from an object file
pub trait Relocatable: Display + Sized {
    // + TryFrom<PathBuf, Error = Error> {
    type Index; // index into the data, e.g. usize

    fn new(origin: PathBuf, object_index: usize) -> Result<Self, Error>;

    fn origin(&self) -> &Path; // not same as section's path since this one's supposed to be cannonical

    fn sections(&self) -> BSI<Self::Index>;

    fn section_data(&self, section_index: Self::Index) -> Result<&[u8], Error>;
}
